/*
 * SYNCHRONY - Modified 2006/09/15 by Maurice Blount
 * based on the program
 *
 * synergy -- mouse and keyboard sharing utility
 * Copyright (C) 2002 Chris Schoeneman
 * 
 * This package is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * found in the file COPYING that should have accompanied this file.
 * 
 * This package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef CSERVERPROXY_H
#define CSERVERPROXY_H

#include "CEvent.h"

class CClient;
class CClientInfo;
class CEventQueueTimer;
class IStreamSYN;

//! Proxy for server
/*!
This class acts a proxy for the server, converting calls into messages
to the server and messages from the server to calls on the client.
*/
class CServerProxy {
public:
	/*!
	Process messages from the server on \p stream and forward to
	\p client.
	*/
	CServerProxy(CClient* client, IStreamSYN* stream);
	~CServerProxy();

	//! @name manipulators
	//@{

	void				onInfoChanged();
	//@}

protected:
	enum EResult { kOkay, kUnknown, kDisconnect };
	EResult				parseHandshakeMessage(const UInt8* code);
	EResult				parseMessage(const UInt8* code);

private:
	
	void				sendInfo(const CClientInfo&);

	void				resetKeepAliveAlarm();
	void				setKeepAliveRate(double);

	// event handlers
	void				handleData(const CEvent&, void*);
	void				handleKeepAliveAlarm(const CEvent&, void*);

	// message handlers
	void				enter();
	void				leave();
	void				resetOptions();
	void				setOptions();
	void				queryInfo();
	void				infoAcknowledgment();
	
private:
	typedef EResult (CServerProxy::*MessageParser)(const UInt8*);

	CClient*			m_client;
	IStreamSYN*			m_stream;

	double				m_keepAliveAlarm;
	CEventQueueTimer*	m_keepAliveAlarmTimer;

	MessageParser		m_parser;
};

#endif
