// ------------------------------------------------------------------------------------------------
// Adder Technology Ltd.
// Communications Port.
// (c) 2005 Adder Technology Ltd,Trafalgar Way, Bar Hill, Cambridge CB3 8SQ.
// Tel: +44 1954 780044 Fax: +44 1954 780081
// ------------------------------------------------------------------------------------------------
// RS232.h
// ------------------------------------------------------------------------------------------------
// Modified:	25.04.05
// By:			Pete Arnold
// Created:		03.03.05
// By:			Pete Arnold
// ------------------------------------------------------------------------------------------------
// History:		25.04.05 Removed local timeout code.
//				03.03.05 Encapsulation of RS232 communications functions and data.
// ------------------------------------------------------------------------------------------------

#ifndef CMSWINDOWSRS232_H
#define CMSWINDOWSRS232_H

#define RS232_IN_QUEUE_SIZE		(1024)		// Input queue size (request).
#define RS232_OUT_QUEUE_SIZE	(1024)		// Output queue size (request).

#define BAUDRATE 19200
#include <windows.h>

typedef unsigned char BYTE;

class CRS232
{
public:
	// Define error codes.
	typedef enum
	{
		RS232_OK,						// OK.
		RS232_CANT_OPEN,				// Couldn't open the COM port.
		RS232_ALREADY_OPEN,				// COM port already open.
		RS232_NOT_OPEN,					// COM port not open.
		RS232_NO_DATA,					// No data to send.
		RS232_FAILED,					// Send or receive failed for undefined reason.
		RS232_TIMEOUT					// Timeout passed and no data received.
	} RS232_RETURN_CODE;

	// Device Control Block definition for serial ports.
	typedef struct
	{
		void*			ComHandle;
		int				ComPortNumber;
		int				Open;
		_DCB			CommsDCB;			// Defined in Windows.
		_COMMTIMEOUTS	CommsTimeouts;		// Defined in Windows.
	} RS232_DCB;

	// Public data structures.
	RS232_DCB			RS232DCB;

	void				RS232Initialise(BYTE comPort, int baudRate);
	RS232_RETURN_CODE	RS232Open(void);
	RS232_RETURN_CODE	RS232Close(void);
	RS232_RETURN_CODE	RS232Send(BYTE* sendBuffer, DWORD* sendBufferSize);
	RS232_RETURN_CODE	RS232ResetSerial(void);
	RS232_RETURN_CODE	UploadCheckCommsPort(int i);
};

#endif