/*
 * SYNCHRONY - Modified 2006/09/15 by Maurice Blount
 * based on the program
 *
 * synergy -- mouse and keyboard sharing utility
 * Copyright (C) 2002 Chris Schoeneman
 * 
 * This package is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * found in the file COPYING that should have accompanied this file.
 * 
 * This package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef CBASECLIENTPROXY_H
#define CBASECLIENTPROXY_H

#include "IClient.h"
#include "CString.h"

//! Generic proxy for client or primary
class CBaseClientProxy : public IClient {
public:
	/*!
	\c name is the name of the client.
	*/
	CBaseClientProxy(const CStringSYN& name);
	~CBaseClientProxy();

	//! @name manipulators
	//@{

	//! Save cursor position
	/*!
	Save the position of the cursor when jumping from client.
	*/
	void				setJumpCursorPos(SInt32 x, SInt32 y);

	//@}
	//! @name accessors
	//@{

	//! Get cursor position
	/*!
	Get the position of the cursor when last jumping from client.
	*/
	void				getJumpCursorPos(SInt32& x, SInt32& y) const;

	//@}

	// IScreen
	virtual void*		getEventTarget() const = 0;
	virtual void		getShape(SInt32& x, SInt32& y,
							SInt32& width, SInt32& height) const = 0;
	
	// IClient overrides
	virtual void		enter(SInt32 xAbs, SInt32 yAbs) = 0;
	virtual bool		leave() = 0;
	virtual void		resetOptions() = 0;
	virtual void		setOptions(const COptionsList& options) = 0;
	virtual CStringSYN	getName() const;

private:
	CStringSYN			m_name;
	SInt32				m_x, m_y;

public:
	// for confirming clients have left their screen when told
	bool				m_hasLeft;
};

#endif
