/*
 * SYNCHRONY - Modified 2006/09/15 by Maurice Blount
 * based on the program
 *
 * synergy -- mouse and keyboard sharing utility
 * Copyright (C) 2002 Chris Schoeneman
 * 
 * This package is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * found in the file COPYING that should have accompanied this file.
 * 
 * This package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef CCLIENTPROXY_H
#define CCLIENTPROXY_H

#include "CBaseClientProxy.h"
#include "CEvent.h"
#include "CString.h"

class IStreamSYN;

//! Generic proxy for client
class CClientProxy : public CBaseClientProxy {
public:
	/*!
	\c name is the name of the client.
	*/
	CClientProxy(const CStringSYN& name, IStreamSYN* adoptedStream);
	~CClientProxy();

	//! @name manipulators
	//@{

	//! Disconnect
	/*!
	Ask the client to disconnect, using \p msg as the reason.
	*/
	void				close(const char* msg);

	//@}
	//! @name accessors
	//@{

	//! Get stream
	/*!
	Returns the stream passed to the c'tor.
	*/
	IStreamSYN*			getStream() const;

	//! Get ready event type
	/*!
	Returns the ready event type.  This is sent when the client has
	completed the initial handshake.  Until it is sent, the client is
	not fully connected.
	*/
	static CEvent::Type	getReadyEvent();

	//! Get disconnect event type
	/*!
	Returns the disconnect event type.  This is sent when the client
	disconnects or is disconnected.  The target is getEventTarget().
	*/
	static CEvent::Type	getDisconnectedEvent();

	//@}

	// IScreen
	virtual void*		getEventTarget() const;
	virtual void		getShape(SInt32& x, SInt32& y,
							SInt32& width, SInt32& height) const = 0;
	virtual void		getCursorPos(SInt32& x, SInt32& y) const = 0;

	// IClient overrides
	virtual void		enter(SInt32 xAbs, SInt32 yAbs) = 0;
	virtual bool		leave() = 0;
	virtual void		resetOptions() = 0;
	virtual void		setOptions(const COptionsList& options) = 0;
private:
	IStreamSYN*			m_stream;
	static CEvent::Type	s_readyEvent;
	static CEvent::Type	s_disconnectedEvent;
};

#endif
