/*
 * SYNCHRONY - Modified 2006/09/15 by Maurice Blount
 * based on the program
 *
 * synergy -- mouse and keyboard sharing utility
 * Copyright (C) 2002 Chris Schoeneman
 * 
 * This package is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * found in the file COPYING that should have accompanied this file.
 * 
 * This package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef CCLIENTPROXY1_0_H
#define CCLIENTPROXY1_0_H

#include "CClientProxy.h"
#include "ProtocolTypes.h"

class CEvent;
class CEventQueueTimer;

//! Proxy for client implementing protocol version 1.0
class CClientProxy1_0 : public CClientProxy {
public:
	CClientProxy1_0(const CStringSYN& name, IStreamSYN* adoptedStream);
	~CClientProxy1_0();

	// IScreen
	virtual void		getShape(SInt32& x, SInt32& y,
							SInt32& width, SInt32& height) const;
	virtual void		getCursorPos(SInt32& x, SInt32& y) const;

	// IClient overrides
	virtual void		enter(SInt32 xAbs, SInt32 yAbs);
	virtual bool		leave();
	virtual void		resetOptions();
	virtual void		setOptions(const COptionsList& options);

protected:
	virtual bool		parseHandshakeMessage(const UInt8* code);
	virtual bool		parseMessage(const UInt8* code);

	virtual void		resetHeartbeatRate();
	virtual void		setHeartbeatRate(double rate, double alarm);
	virtual void		resetHeartbeatTimer();
	virtual void		addHeartbeatTimer();
	virtual void		removeHeartbeatTimer();

private:
	void				disconnect();
	void				removeHandlers();

	CBaseClientProxy*	mapToNeighbor(CBaseClientProxy* src,
						EDirection srcSide, SInt32& x, SInt32& y);

	void				handleData(const CEvent&, void*);
	void				handleDisconnect(const CEvent&, void*);
	void				handleWriteError(const CEvent&, void*);
	void				handleFlatline(const CEvent&, void*);

	void				checkJumpZone(CStringSYN, SInt32, SInt32, SInt8);
	
	bool				recvInfo();
	
private:
	typedef bool (CClientProxy1_0::*MessageParser)(const UInt8*);
	
	CClientInfo			m_info;
	double				m_heartbeatAlarm;
	CEventQueueTimer*	m_heartbeatTimer;
	MessageParser		m_parser;
};

#endif
