/*
 * SYNCHRONY - Modified 2006/09/15 by Maurice Blount
 * based on the program
 *
 * synergy -- mouse and keyboard sharing utility
 * Copyright (C) 2002 Chris Schoeneman
 * 
 * This package is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * found in the file COPYING that should have accompanied this file.
 * 
 * This package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include "CPrimaryClient.h"
#include "CScreen.h"
#include "CLog.h"

//
// CPrimaryClient
//

CPrimaryClient::CPrimaryClient(const CStringSYN& name, CScreen* screen) :
	CBaseClientProxy(name),
	m_screen(screen)
{
	m_screen->enter();
}

CPrimaryClient::~CPrimaryClient()
{
	// do nothing
}

void
CPrimaryClient::reconfigure(UInt32 activeSides)
{
	m_screen->reconfigure(activeSides);
}

SInt32
CPrimaryClient::getJumpZoneSize() const
{
	return m_screen->getJumpZoneSize();
}

void
CPrimaryClient::getCursorCenter(SInt32& x, SInt32& y) const
{
	m_screen->getCursorCenter(x, y);
}

bool
CPrimaryClient::isLockedToScreen() const
{
	return m_screen->isLockedToScreen();
}

void*
CPrimaryClient::getEventTarget() const
{
	return m_screen->getEventTarget();
}

void
CPrimaryClient::getShape(SInt32& x, SInt32& y,
				SInt32& width, SInt32& height) const
{
	m_screen->getShape(x, y, width, height);
}

void
CPrimaryClient::getCursorPos(SInt32& x, SInt32& y) const
{
	m_screen->getCursorPos(x, y);
}

void
CPrimaryClient::enable()
{
	m_screen->enable();
}

void
CPrimaryClient::disable()
{
	m_screen->disable();
}

void
CPrimaryClient::enter(SInt32 xAbs, SInt32 yAbs)
{
	m_screen->warpCursor(xAbs, yAbs);
	m_screen->enter();
}

bool
CPrimaryClient::leave()
{
	return m_screen->leave();
}

void
CPrimaryClient::resetOptions()
{
	m_screen->resetOptions();
}

void
CPrimaryClient::setOptions(const COptionsList& options)
{
	m_screen->setOptions(options);
}
