/*
 * SYNCHRONY - Modified 2006/09/15 by Maurice Blount
 * based on the program
 *
 * synergy -- mouse and keyboard sharing utility
 * Copyright (C) 2002 Chris Schoeneman
 * 
 * This package is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * found in the file COPYING that should have accompanied this file.
 * 
 * This package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef ICLIENT_H
#define ICLIENT_H

#include "IScreen.h"
#include "MouseTypes.h"
#include "OptionTypes.h"
#include "CString.h"

//! Client interface
/*!
This interface defines the methods necessary for the server to
communicate with a client.
*/
class IClient : public IScreen {
public:
	//! @name manipulators
	//@{

	//! Enter screen
	/*!
	Enter the screen.  The cursor should be warped to \p xAbs,yAbs.
	*/
	virtual void		enter(SInt32 xAbs, SInt32 yAbs) = 0;

	//! Leave screen
	/*!
	Leave the screen.  Return false iff the user may not leave the
	client's screen (because, for example, a button is down).
	*/
	virtual bool		leave() = 0;

	//! Notify of options changes
	/*!
	Reset all options to their default values.
	*/
	virtual void		resetOptions() = 0;

	//! Notify of options changes
	/*!
	Set options to given values.  Ignore unknown options and don't
	modify our options that aren't given in \c options.
	*/
	virtual void		setOptions(const COptionsList& options) = 0;

	//@}
	//! @name accessors
	//@{

	//! Get client name
	/*!
	Return the client's name.
	*/
	virtual CStringSYN		getName() const = 0;

	//@}

	// IScreen overrides
	virtual void*		getEventTarget() const = 0;
	virtual void		getShape(SInt32& x, SInt32& y,
							SInt32& width, SInt32& height) const = 0;
	virtual void		getCursorPos(SInt32& x, SInt32& y) const = 0;
};

#endif
