/*
 * SYNCHRONY - Modified 2006/09/15 by Maurice Blount
 * based on the program
 *
 * synergy -- mouse and keyboard sharing utility
 * Copyright (C) 2003 Chris Schoeneman
 * 
 * This package is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * found in the file COPYING that should have accompanied this file.
 * 
 * This package is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef ISCREEN_H
#define ISCREEN_H

#include "IInterface.h"
#include "MouseTypes.h"
#include "CEvent.h"


//! Screen interface
/*!
This interface defines the methods common to all screens.
*/
class IScreen : public IInterface {
public:
	
	//! @name accessors
	//@{

	//! Get event target
	/*!
	Returns the target used for events created by this object.
	*/
	virtual void*		getEventTarget() const = 0;

	//! Get screen shape
	/*!
	Return the position of the upper-left corner of the screen in \c x and
	\c y and the size of the screen in \c width and \c height.
	*/
	virtual void		getShape(SInt32& x, SInt32& y,
							SInt32& width, SInt32& height) const = 0;

	//! Get cursor position
	/*!
	Return the current position of the cursor in \c x and \c y.
	*/
	virtual void		getCursorPos(SInt32& x, SInt32& y) const = 0;

	//! Get motion event type
	/*!
	Returns the mouse moved event type.  This is sent whenever the screen's
	mouse is moved by the user.
	*/
	static CEvent::Type getMouseMoveEvent();

	//! Get error event type
	/*!
	Returns the error event type.  This is sent whenever the screen has
	failed for some reason (e.g. the X Windows server died).
	*/
	static CEvent::Type	getErrorEvent();

	//! Get shape changed event type
	/*!
	Returns the shape changed event type.  This is sent whenever the
	screen's shape changes.
	*/
	static CEvent::Type	getShapeChangedEvent();

	//! Get suspend event type
	/*!
	Returns the suspend event type. This is sent whenever the system goes
	to sleep or a user session is deactivated (fast user switching).
	*/
	static CEvent::Type	getSuspendEvent();
	
	//! Get resume event type
	/*!
	Returns the suspend event type. This is sent whenever the system wakes
	up or a user session is activated (fast user switching).
	*/
	static CEvent::Type	getResumeEvent();
	
	//@}

	// moved from IPrimaryScreen

	//! Button event data
	class CButtonInfo {
	public:
		static CButtonInfo* alloc(ButtonID);
		static CButtonInfo* alloc(const CButtonInfo&);

		static bool			equal(const CButtonInfo*, const CButtonInfo*);

	public:
		ButtonID		m_button;
	};
	//! Motion event data
	class CMotionInfo {
	public:
		static CMotionInfo* alloc(SInt32 x, SInt32 y);

	public:
		SInt32			m_x;
		SInt32			m_y;
	};
private:
	static CEvent::Type s_mouseMoveEvent;
	static CEvent::Type	s_errorEvent;
	static CEvent::Type	s_shapeChangedEvent;
	static CEvent::Type	s_suspendEvent;
	static CEvent::Type	s_resumeEvent;	
};

#endif
